@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_core.c,v 1.12 2001/01/26 10:19:23 wensong Exp $
+ * Version:     $Id: ip_vs_core.c,v 1.13 2001/01/29 15:27:27 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -500,6 +500,13 @@ static int ip_vs_leave(struct ip_vs_service *svc, struct sk_buff *skb)
 }
 
 
+/*
+ *      It is hooked before NF_IP_PRI_NAT_SRC at the NF_IP_POST_ROUTING
+ *      chain, and is used for VS/NAT.
+ *      It detects packets for VS/NAT connections and sends the packets
+ *      immediately. This can avoid that iptable_nat mangles the packets
+ *      for VS/NAT. 
+ */
 static unsigned int ip_vs_post_routing(unsigned int hooknum,
                                        struct sk_buff **skb_p,
                                        const struct net_device *in,
@@ -517,6 +524,7 @@ static unsigned int ip_vs_post_routing(unsigned int hooknum,
 	return NF_STOLEN;
 }
 
+
 /*
  *	Handle ICMP messages in the inside-to-outside direction (outgoing).
  *	Find any that might be relevant, check against existing connections,
@@ -540,11 +548,10 @@ static int ip_vs_out_icmp(struct sk_buff **skb_p)
 
 	/*
 	 * Work through seeing if this is for us.
-	 * These checks are supposed to be in an order that
-	 * means easy things are checked first to speed up
-	 * processing.... however this means that some
-	 * packets will manage to get a long way down this
-	 * stack and then be rejected, but thats life
+	 * These checks are supposed to be in an order that means easy
+         * things are checked first to speed up processing.... however
+         * this means that some packets will manage to get a long way
+         * down this stack and then be rejected, but that's life.
 	 */
 	if ((icmph->type != ICMP_DEST_UNREACH) &&
 	    (icmph->type != ICMP_SOURCE_QUENCH) &&
@@ -772,7 +779,7 @@ static unsigned int ip_vs_out(unsigned int hooknum,
                   ip_vs_proto_name(iph->protocol), size, csum);
 
 	/*
-	 * 	Protocol csum
+	 * 	Protocol checksum
 	 */
 	switch (iph->protocol) {
         case IPPROTO_TCP:
@@ -808,6 +815,12 @@ static unsigned int ip_vs_out(unsigned int hooknum,
 }
 
 
+/*
+ *      Check if the packet is for VS/NAT connections, then send it
+ *      immediately.
+ *      Called by ip_fw_compact to detect packets for VS/NAT before
+ *      they are changed by ipchains masquerading code.
+ */
 unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
                                  int (*okfn)(struct sk_buff *))
 {
@@ -817,6 +830,8 @@ unsigned int check_for_ip_vs_out(struct sk_buff **skb_p,
         if (ret != NF_ACCEPT) {
                 return ret;
         } else {
+                /* send the packet immediately if it is already mangled
+                   by ip_vs_out */
                 if ((*skb_p)->nfmark == NFMARK_IPVS_PROPERTY) {
                         (*okfn)(*skb_p);
                         return NF_STOLEN;
@@ -1109,6 +1124,30 @@ static unsigned int ip_vs_in(unsigned int hooknum,
 }
 
 
+/*
+ *	It is hooked at the NF_IP_FORWARD chain, in order to catch ICMP
+ *      packets destined for 0.0.0.0/0.
+ *      When fwmark-based virtual service is used, such as transparent
+ *      cache cluster, TCP packets can be marked and routed to ip_vs_in,
+ *      but ICMP destined for 0.0.0.0/0 cannot not be easily marked and
+ *      sent to ip_vs_in_icmp. So, catch them at the NF_IP_FORWARD chain
+ *      and send them to ip_vs_in_icmp.
+ */
+static unsigned int ip_vs_forward_icmp(unsigned int hooknum,
+                                       struct sk_buff **skb_p,
+                                       const struct net_device *in,
+                                       const struct net_device *out,
+                                       int (*okfn)(struct sk_buff *))
+{
+ 	struct iphdr	*iph   = (*skb_p)->nh.iph;
+
+	if (iph->protocol == IPPROTO_ICMP)
+		return ip_vs_in_icmp(skb_p);
+
+	return NF_ACCEPT;
+}
+
+
 /* After packet filtering, forward packet through VS/DR, VS/TUN,
    or VS/NAT(change destination), so that filtering rules can be
    applied to IPVS. */
@@ -1123,6 +1162,13 @@ static struct nf_hook_ops ip_vs_out_ops = {
         ip_vs_out, PF_INET, NF_IP_FORWARD, 100
 };
 
+/* After packet filtering (but before ip_vs_out_icmp), catch icmp
+   destined for 0.0.0.0/0, which is for incoming IPVS connections */
+static struct nf_hook_ops ip_vs_forward_icmp_ops = {
+        { NULL, NULL },
+        ip_vs_forward_icmp, PF_INET, NF_IP_FORWARD, 99
+};
+
 /* Before the netfilter connection tracking, exit from POST_ROUTING */
 static struct nf_hook_ops ip_vs_post_routing_ops = {
         { NULL, NULL },
@@ -1172,10 +1218,17 @@ static int __init ip_vs_init(void)
 		IP_VS_ERR("can't register post_routing hook.\n");
 		goto cleanup_outops;
 	}
+	ret = nf_register_hook(&ip_vs_forward_icmp_ops);
+	if (ret < 0) {
+		IP_VS_ERR("can't register forward_icmp hook.\n");
+		goto cleanup_postroutingops;
+	}
 
         IP_VS_INFO("ipvs loaded.\n");
 	return ret;
 
+  cleanup_postroutingops:
+	nf_unregister_hook(&ip_vs_post_routing_ops);
   cleanup_outops:
 	nf_unregister_hook(&ip_vs_out_ops);
   cleanup_inops:
