@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.10 2001/01/15 13:43:46 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.11 2001/01/29 15:27:27 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -371,36 +371,39 @@ ip_vs_service_get(unsigned long fwmark,
 
 	if (fwmark) {
 		/*	
-		 *	Check the table hashed by fwmark
+		 *	Check the table hashed by fwmark first
 		 */
 		svc = __ip_vs_svc_fwm_get(fwmark);
-	} else {
-		/*	
-		 *	Check the table hashed by <protocol,addr,port>
-		 *	first for "full" addressed entries
-		 */
-		svc = __ip_vs_service_get(protocol, vaddr, vport);
-
-		if (svc == NULL
-                    && protocol == IPPROTO_TCP
-                    && atomic_read(&ip_vs_ftpsvc_counter)
-                    && (vport == FTPDATA || ntohs(vport) >= PROT_SOCK)) {
-			/*
-			 * Check if ftp service entry exists, the packet
-			 * might belong to FTP data connections.
-			 */
-			svc = __ip_vs_service_get(protocol, vaddr, FTPPORT);
-		}
-
-		if (svc == NULL
-                    && atomic_read(&ip_vs_nullsvc_counter)) {
-			/*
-			 * Check if the catch-all port (port zero) exists
-			 */
-			svc = __ip_vs_service_get(protocol, vaddr, 0);
-		}
+                if (svc)
+                        goto out;
 	}
+        
+        /*	
+         *	Check the table hashed by <protocol,addr,port>
+         *	for "full" addressed entries
+         */
+        svc = __ip_vs_service_get(protocol, vaddr, vport);
 
+        if (svc == NULL
+            && protocol == IPPROTO_TCP
+            && atomic_read(&ip_vs_ftpsvc_counter)
+            && (vport == FTPDATA || ntohs(vport) >= PROT_SOCK)) {
+                /*
+                 * Check if ftp service entry exists, the packet
+                 * might belong to FTP data connections.
+                 */
+                svc = __ip_vs_service_get(protocol, vaddr, FTPPORT);
+        }
+
+        if (svc == NULL
+            && atomic_read(&ip_vs_nullsvc_counter)) {
+                /*
+                 * Check if the catch-all port (port zero) exists
+                 */
+                svc = __ip_vs_service_get(protocol, vaddr, 0);
+        }
+
+  out:
 	read_unlock(&__ip_vs_svc_lock);
 
         IP_VS_DBG(6, "lookup service: fwm %ld %s %u.%u.%u.%u:%u %s\n",
@@ -893,7 +896,7 @@ static int ip_vs_del_dest(struct ip_vs_service *svc,struct ip_vs_rule_user *ur)
 	write_unlock_bh(&__ip_vs_svc_lock);
 
 	/*
-	 *	Delete dest
+	 *	Delete the destination
 	 */
 	__ip_vs_del_dest(dest);
 
