@@ -1,7 +1,7 @@
 /*
  * IPVS:        Locality-Based Least-Connection with Replication scheduler
  *
- * Version:     $Id: ip_vs_lblcr.c,v 1.3 2001/01/08 14:08:53 wensong Exp $
+ * Version:     $Id: ip_vs_lblcr.c,v 1.4 2001/01/29 15:27:27 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@gnuchina.org>
  *
@@ -11,6 +11,8 @@
  *              2 of the License, or (at your option) any later version.
  *
  * Changes:
+ *     Julian Anastasov        :    Added the missing (dest->weight>0)
+ *                                  condition in the ip_vs_dest_set_max.
  *
  */
 
@@ -185,7 +187,8 @@ static inline struct ip_vs_dest *ip_vs_dest_set_min(struct ip_vs_dest_set *set)
         /* select the first destination server, whose weight > 0 */
         for (e=set->list; e!=NULL; e=e->next) {
                 least = e->dest;
-                if (atomic_read(&least->weight) > 0) {
+                if ((atomic_read(&least->weight) > 0)
+                    && (least->flags & IP_VS_DEST_F_AVAILABLE)) {
                         loh = atomic_read(&least->activeconns) * 50
                                 + atomic_read(&least->inactconns);
                         goto nextstage;
@@ -200,8 +203,9 @@ static inline struct ip_vs_dest *ip_vs_dest_set_min(struct ip_vs_dest_set *set)
                 dest = e->dest;
                 doh = atomic_read(&dest->activeconns) * 50
                         + atomic_read(&dest->inactconns);
-                if (loh * atomic_read(&dest->weight) >
-                    doh * atomic_read(&least->weight)) {
+                if ((loh * atomic_read(&dest->weight) >
+                     doh * atomic_read(&least->weight))
+                    && (dest->flags & IP_VS_DEST_F_AVAILABLE)) {
                         least = dest;
                         loh = doh;
                 }
@@ -248,8 +252,9 @@ static inline struct ip_vs_dest *ip_vs_dest_set_max(struct ip_vs_dest_set *set)
                 doh = atomic_read(&dest->activeconns) * 50
                         + atomic_read(&dest->inactconns);
                 /* moh/mw < doh/dw ==> moh*dw < doh*mw, where mw,dw>0 */ 
-                if (moh * atomic_read(&dest->weight) <
-                    doh * atomic_read(&most->weight)) {
+                if ((moh * atomic_read(&dest->weight) <
+                     doh * atomic_read(&most->weight))
+                    && (atomic_read(&dest->weight) > 0)) {
                         most = dest;
                         moh = doh;
                 }
@@ -816,8 +821,9 @@ ip_vs_lblcr_schedule(struct ip_vs_service *svc, struct iphdr *iph)
                 ip_vs_lblcr_hash(tbl, en);
         } else {
                 dest = ip_vs_dest_set_min(&en->set);
-                if (!(dest->flags & IP_VS_DEST_F_AVAILABLE)
-                    || is_overloaded(dest, svc)) {
+		if (!dest)
+			return NULL;
+                if (is_overloaded(dest, svc)) {
                         dest = __ip_vs_wlc_schedule(svc, iph);
                         if (dest == NULL) {
                                 IP_VS_DBG(1, "no destination available\n");
@@ -829,7 +835,8 @@ ip_vs_lblcr_schedule(struct ip_vs_service *svc, struct iphdr *iph)
                     jiffies-en->set.lastmod > sysctl_ip_vs_lblcr_expiration) {
                         struct ip_vs_dest *m;
                         m = ip_vs_dest_set_max(&en->set);
-                        ip_vs_dest_set_erase(&en->set, m);
+                        if (m)
+                                ip_vs_dest_set_erase(&en->set, m);
                 }
         }
         en->lastuse = jiffies;
